'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
/* eslint import/prefer-default-export: off */

var createSSRContext = exports.createSSRContext = function createSSRContext(settings) {
    var awaitStack = [];
    var callbacks = {};

    var rootUrl = settings.rootUrl ? settings.rootUrl : '/';

    var _apiUrl = settings.apiUrl ? settings.apiUrl : '/api';

    var checkTimer = null;

    var upsert = function upsert(action) {
        if (!callbacks[action]) {
            callbacks[action] = [];
        }
    };

    var emit = function emit(action) {
        upsert(action);
        callbacks[action].forEach(function (ticket) {
            if (ticket.once) {
                ticket.off();
            }
            ticket.fn();
        });
    };

    var checkStack = function checkStack() {
        if (awaitStack.length <= 0) {
            emit('complete');
        }
        return awaitStack.length;
    };

    var unsubscribe = function unsubscribe(action, ticket) {
        var idx = callbacks[action].indexOf(ticket);
        callbacks[action].splice(idx, 1);
    };

    var subscribe = function subscribe(action, fn) {
        var once = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

        upsert(action);
        var ticket = { fn: fn, once: once };
        ticket.off = function () {
            return unsubscribe(action, ticket);
        };

        callbacks[action].push(ticket);
        setTimeout(checkStack);
        return ticket;
    };

    var reducer = {
        checkStack: checkStack,
        on: function on(action, fn) {
            return subscribe(action, fn, false);
        },
        once: function once(action, fn) {
            return subscribe(action, fn, true);
        },
        await: function _await(p) {
            awaitStack.push(p);

            p.then(function (v) {
                clearTimeout(checkTimer);
                var idx = awaitStack.indexOf(p);
                awaitStack.splice(idx, 1);
                checkTimer = setTimeout(checkStack);
            }, function (e) {
                clearTimeout(checkTimer);
                var idx = awaitStack.indexOf(p);
                awaitStack.splice(idx, 1);
                checkTimer = setTimeout(checkStack);
            });

            return p;
        },
        apiUrl: function apiUrl(url) {
            console.warn('[create-ssr-context] apiUrl is deprecated, please use "getApiUrl()" instead');
            return '' + _apiUrl + url;
        },
        getRootUrl: function getRootUrl(url) {
            return '' + rootUrl + url;
        },
        getApiUrl: function getApiUrl(url) {
            return '' + _apiUrl + url;
        },
        getRequestHandler: function getRequestHandler() {
            return settings.req;
        },
        getResponsetHandler: function getResponsetHandler() {
            return settings.res;
        }
    };

    return {
        reducers: {
            ssr: function ssr() {
                return reducer;
            }
            // ssr: state => state || reducer,
        }
    };
};