'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createSSRRender = exports.renderAppWithState = exports.renderApp = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactLoadable = require('react-loadable');

var _reactLoadable2 = _interopRequireDefault(_reactLoadable);

var _server = require('react-dom/server');

var _createMemoryHistory = require('history/createMemoryHistory');

var _createMemoryHistory2 = _interopRequireDefault(_createMemoryHistory);

var _currentVersion = require('./current-version');

var _currentVersion2 = _interopRequireDefault(_currentVersion);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
- initialState:
  - ssr
    - serverUrl: http://localhost:8080
    - apiUrl: http://localhost:8080/api
    - request: express request object

- settings
  - timeout: 5000
  - userAgent: 'string'
*/

// @TODO: handle react-loadable
var renderApp = exports.renderApp = function renderApp(_ref) {
    var Root = _ref.Root,
        location = _ref.location,
        context = _ref.context,
        initialState = _ref.initialState;
    return {
        html: (0, _server.renderToString)(_react2.default.createElement(Root, (0, _extends3.default)({}, initialState, {
            location: location
        }))),
        initialState: initialState,
        context: context,
        modules: []
    };
};

var renderAppWithState = exports.renderAppWithState = function renderAppWithState(_ref2) {
    var Root = _ref2.Root,
        store = _ref2.store,
        location = _ref2.location,
        context = _ref2.context,
        timeout = _ref2.timeout,
        limit = _ref2.limit;
    return new Promise(function (resolve, reject) {
        var _store$getState = store.getState(),
            ssr = _store$getState.ssr;

        var modules = [];
        var rerenderCount = 0;

        var timer = setTimeout(function () {
            return reject('timeout');
        }, timeout);

        var registerModule = function registerModule(moduleName) {
            if (modules.indexOf(moduleName) === -1) {
                modules.push(moduleName);
            }
        };

        var tick = function () {
            var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
                var previousLength, html;
                return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                        switch (_context.prev = _context.next) {
                            case 0:
                                if (!(rerenderCount >= limit)) {
                                    _context.next = 5;
                                    break;
                                }

                                clearTimeout(timer);
                                return _context.abrupt('return', reject('limit'));

                            case 5:
                                rerenderCount += 1;

                            case 6:

                                // run the static render
                                previousLength = modules.length;
                                html = (0, _server.renderToString)(_react2.default.createElement(
                                    _reactLoadable2.default.Capture,
                                    { report: registerModule },
                                    _react2.default.createElement(Root, {
                                        store: store,
                                        location: location,
                                        context: context
                                    })
                                ));

                                // await for asynchronous modules loading to complete

                                if (modules.length > previousLength) {
                                    ssr.await(_reactLoadable2.default.preloadAll());
                                }

                                // check the completeness of the rendering loop
                                if (!ssr.checkStack()) {
                                    clearTimeout(timer);
                                    resolve({ html: html, modules: modules, context: context });
                                } else {
                                    ssr.once('complete', tick);
                                }

                            case 10:
                            case 'end':
                                return _context.stop();
                        }
                    }
                }, _callee, undefined);
            }));

            return function tick() {
                return _ref3.apply(this, arguments);
            };
        }();

        tick();
    });
};

var createSSRRender = exports.createSSRRender = function createSSRRender(Root) {
    var _ref4 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
        createState = _ref4.createState;

    var ssrRender = function () {
        var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(location, initialState, settings) {
            var history, state, rendered;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            if (createState) {
                                _context2.next = 4;
                                break;
                            }

                            _context2.next = 3;
                            return renderApp({
                                Root: Root,
                                location: location,
                                initialState: initialState,
                                context: settings.context
                            });

                        case 3:
                            return _context2.abrupt('return', _context2.sent);

                        case 4:

                            // setup the application state, history is not used but is
                            // needed for the correct working of the create state thingy
                            history = (0, _createMemoryHistory2.default)();
                            _context2.next = 7;
                            return createState(initialState, history);

                        case 7:
                            state = _context2.sent;


                            // render the app awaiting for async calls and react-loadable things
                            rendered = null;
                            _context2.prev = 9;
                            _context2.next = 12;
                            return renderAppWithState({
                                Root: Root,
                                location: location,
                                store: state.store,
                                context: settings.context,
                                timeout: settings.timeout,
                                limit: settings.limit
                            });

                        case 12:
                            rendered = _context2.sent;
                            _context2.next = 19;
                            break;

                        case 15:
                            _context2.prev = 15;
                            _context2.t0 = _context2['catch'](9);

                            console.log('error', _context2.t0.message);
                            rendered = {
                                html: '',
                                context: {},
                                modules: []
                            };

                        case 19:
                            return _context2.abrupt('return', (0, _extends3.default)({}, rendered, {
                                initialState: state.store ? state.store.getState() : initialState
                            }));

                        case 20:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, undefined, [[9, 15]]);
        }));

        return function ssrRender(_x2, _x3, _x4) {
            return _ref5.apply(this, arguments);
        };
    }();

    // the version is used to check if server and client are aligned with the same library
    ssrRender.ssrVersion = _currentVersion2.default;
    return ssrRender;
};