'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createSSRRouter = undefined;

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _createSsrRegisterHook = require('./create-ssr-register-hook');

var _serveAppStatic = require('./serve-app-static.middleware');

var _serveAppSsr = require('./serve-app-ssr.middleware');

var _serveBuild = require('./serve-build.middleware');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var createSSRRouter = exports.createSSRRouter = function createSSRRouter(settings) {
    var router = _express2.default.Router();

    // handle require images
    if (settings.ssrEnabled === 'yes') {
        (0, _createSsrRegisterHook.createSSRRegisterHook)();
    }

    // serve client app
    var serveApp = settings.ssrEnabled === 'yes' ? _serveAppSsr.serveAppSSR : _serveAppStatic.serveAppStatic;

    // Optional blacklist out from ssr
    if (settings.ssrEnabled === 'yes' && settings.ssrBlacklist) {
        var rules = Array.isArray(settings.ssrBlacklist) ? settings.ssrBlacklist : settings.ssrBlacklist.split(',');
        rules.forEach(function (rule) {
            return router.get(rule, (0, _serveAppStatic.serveAppStatic)(settings));
        });
    }

    // Default routes handling
    router.get('/', serveApp(settings));
    router.use((0, _serveBuild.serveBuild)(settings));
    router.get('*', serveApp(settings));

    return router;
};