'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.serveAppSSR = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _currentVersion = require('./current-version');

var _currentVersion2 = _interopRequireDefault(_currentVersion);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var fs = require('fs');

var _require = require('react-helmet'),
    Helmet = _require.Helmet;

var _require2 = require('react-loadable/webpack'),
    getBundles = _require2.getBundles;

var readFile = function readFile(filePath) {
    var encoding = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'utf8';
    return new Promise(function (resolve, reject) {
        if (readFile.cache[filePath]) {
            return resolve(readFile.cache[filePath]);
        }

        try {
            fs.readFile(filePath, encoding, function (err, data) {
                if (err) {
                    reject(err);
                    return;
                }
                readFile.cache[filePath] = data;
                resolve(data);
            });
        } catch (err) {
            reject(err);
        }
    });
};

// setup file cache memoization
readFile.cache = {};

var prepHTML = function prepHTML(template, _ref) {
    var html = _ref.html,
        head = _ref.head,
        body = _ref.body,
        state = _ref.state,
        jsBundles = _ref.jsBundles,
        cssBundles = _ref.cssBundles,
        ssrDisableJs = _ref.ssrDisableJs,
        ssrUseWebpackJs = _ref.ssrUseWebpackJs,
        nodeEnv = _ref.nodeEnv;

    var data = template;
    data = data.replace('<html lang="en">', '<html ' + html + '>');
    data = data.replace('</head>', '' + head + cssBundles.join('') + '</head>');

    // avoid to send out the redux state if client js is disabled
    if (ssrDisableJs !== 'yes') {
        // eslint-disable-next-line
        data = data.replace('<div id="root"></div>', '<div id="root"></div><script>window.REDUX_INITIAL_STATE = ' + JSON.stringify(state) + ';</script>');
    }

    // react-loadable -- inject chunks
    if (ssrDisableJs !== 'yes') {
        data = data.replace('<script src="/static/js/main.', jsBundles.join('') + '<script type="text/javascript" src="/static/js/main.');
    }

    // inject the pre-rendered body of the application
    data = data.replace('<div id="root"></div>', '<div id="root">' + body + '</div>');

    // Use bundles from development website (experimental)
    if (nodeEnv === 'development' && ssrUseWebpackJs === 'yes') {
        data = data.replace(/<link href="\/static\/css\/main.([^\s]*).css" rel="stylesheet">/g, '');
        data = data.replace(/\/static\/js\/main.([^\s]*).js/g, '//localhost:3000/static/js/bundle.js');
    }

    // remove bundle js (dev, experimental)
    if (ssrDisableJs === 'yes') {
        data = data.replace(/<script src="\/static\/js\/([^\s]*).js"><\/script>/g, '');
        data = data.replace(/<script.*>.*<\/script>/g, '');
    }

    return data;
};

/**
 * Settings:
 * - ssrRoot (string) - client app source folder absolute path
 * - ssrBuild (string) - client app build folder absolute path
 * - ssrPort: (string) - ssr server port for default api calls
 * - ssrTimeout: (int) - rendering timeout in milliseconds
 * - ssrDisableJs: (string)[yes|no]
 * - ssrUseDynamicJs: (string)[yes|no]
 * - nodeEnv: (string)[development|production]
 */
var staticRender = null;
var serveAppSSR = exports.serveAppSSR = function serveAppSSR() {
    var settings = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    return function () {
        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(req, res, next) {
            var filePath, htmlTemplate, initialState, prerender, helmet, jsBundles, cssBundles, bundleStats;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            _context.prev = 0;

                            console.log('[ssr] ' + req.url);

                            // try to import the staticRender function from the app's source code

                            if (staticRender) {
                                _context.next = 16;
                                break;
                            }

                            _context.prev = 3;

                            staticRender = require(path.join(settings.ssrRoot, 'app/ssr')).staticRender;

                            if (staticRender) {
                                _context.next = 7;
                                break;
                            }

                            throw new Error('ssr static render was not provided');

                        case 7:
                            if (!(typeof staticRender !== 'function')) {
                                _context.next = 9;
                                break;
                            }

                            throw new Error('ssr static render does not appear to be a function');

                        case 9:
                            if (!(staticRender.ssrVersion !== _currentVersion2.default)) {
                                _context.next = 11;
                                break;
                            }

                            throw new Error('ssr static render version signature does not appear to match');

                        case 11:
                            _context.next = 16;
                            break;

                        case 13:
                            _context.prev = 13;
                            _context.t0 = _context['catch'](3);

                            // fallback on a default static render that will visualize the error
                            staticRender = function staticRender() {
                                return {
                                    html: _context.t0.message,
                                    initialState: {},
                                    context: {}
                                };
                            };

                        case 16:
                            filePath = path.resolve(path.join(settings.ssrBuild, 'index.html'));
                            _context.next = 19;
                            return readFile(filePath);

                        case 19:
                            htmlTemplate = _context.sent;
                            initialState = {
                                app: {
                                    name: 'server'
                                },
                                ssr: {
                                    // req, res, // proxy express handlers
                                    rootUrl: 'http://localhost:' + settings.ssrPort,
                                    apiUrl: 'http://localhost:' + settings.ssrPort + '/api'
                                }

                                // render app in order to get the initialState
                            };
                            _context.next = 23;
                            return staticRender(req.url, initialState, {
                                req: req, res: res, // proxy express handlers
                                context: {},
                                timeout: settings.ssrTimeout, // max rendering duration
                                limit: 10 // max re-rendering operations
                            });

                        case 23:
                            prerender = _context.sent;


                            // get SEO headers for the page
                            helmet = Helmet.renderStatic();

                            // react-loadable -- get rendered chunks, if eny exists

                            jsBundles = [];
                            cssBundles = [];

                            try {
                                bundleStats = require(path.join(settings.ssrBuild, 'react-loadable.json'));

                                jsBundles = getBundles(bundleStats, prerender.modules).filter(function (bundle) {
                                    return bundle.file.indexOf('.js.map') === -1;
                                }) // remove sourcemaps
                                .filter(function (bundle) {
                                    return bundle.file.indexOf('.css') === -1;
                                }) // remove css
                                .map(function (bundle) {
                                    return '<script type="text/javascript" src="' + bundle.publicPath + '"></script>';
                                });
                                cssBundles = getBundles(bundleStats, prerender.modules).filter(function (bundle) {
                                    return bundle.file.indexOf('.css') !== -1;
                                }) // remove css
                                .filter(function (bundle) {
                                    return bundle.file.indexOf('.map') === -1;
                                }) // remove maps
                                .map(function (bundle) {
                                    return '<link href="' + bundle.publicPath + '" rel="stylesheet">';
                                });
                            } catch (err) {}

                            // handle simple redirect
                            // @TODO: implement status
                            // if (prerender.context.action === 'REPLACE') {
                            //     res.redirect(prerender.context.url)
                            //     return
                            // }

                            res.send(prepHTML(htmlTemplate, (0, _extends3.default)({}, settings, {
                                html: helmet.htmlAttributes.toString(),
                                head: [helmet.title.toString(), helmet.meta.toString(), helmet.link.toString()].join(''),
                                body: prerender.html,
                                state: prerender.initialState,
                                jsBundles: jsBundles,
                                cssBundles: cssBundles
                            })));
                            _context.next = 34;
                            break;

                        case 31:
                            _context.prev = 31;
                            _context.t1 = _context['catch'](0);

                            next(_context.t1);

                        case 34:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, undefined, [[0, 31], [3, 13]]);
        }));

        return function (_x3, _x4, _x5) {
            return _ref2.apply(this, arguments);
        };
    }();
};